unit UBill; 

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, UCore, USqlDatabase, UDatabase, UUser, dateutils;

const
  SpecificSymbol: Integer = 1;
  FileFolder: string = 'doklady';
  FileNamePrefix: string = 'doklad-';

type

  { TBillItem }

  TBillItem = class(TDbObject)
    Description: string;
    Quantity: Integer;
    Price: Double;
    procedure Load;  override;
    procedure Store; override;
  end;

  TBillItemList = class(TDbList)
  end;

  { TBill }

  TBill = class(TDbObject)
  private
    function CreateBill: Integer;
  public
    Items: TBillItemList;
    User: TUser;
    UserId: Integer;
    BillCode: string;
    TimeCreate: TDateTime;
    TimeTo: TDateTime;
    TimeFrom: TDateTime;
    TimeDue: TDateTime;
    function GenerateBill: string;
    procedure Load; override;
    procedure Store; override;
  end;

implementation

{ TBill }

function TBill.GenerateBill: string;
var
  DbRows: TDbRows;
  BillItem: TBillItem;
  Total: Real;
  I: Integer;
  T: TUser;
begin
  T.FirstName:= 'ss';

  DbRows := Database.Select('finance_bills', '*', 'id=' + IntToStr(Id));
  LoadFromDbRecord(DbRows[0]);
  DbRows.Free;

  DbRows := Database.Select('users', '*', 'id=' + IntToStr(UserId));
  User := TUser.Create;
  User.LoadFromDbRecord(DbRows[0]);
  DbRows.Free;

  DbRows := Database.Select('finance_bills_items', '*', 'bill_id=' + IntToStr(Id));
  for I := 0 to DbRows.Count - 1 do begin
    BillItem := TBillItem.Create;
//    BillItem.LoadFromDbRecord(DbRows[I]);
    Items.Add(BillItem);
  end;


  Result := '<table width="100%"><tr><td colspan="2">' +
    '<font size="6"><div align="center">Faktura - daňový doklad</font></div>' +
    '<hr></td></tr>' +
    '<tr><td valign="top" width="50%"><strong>Dodavatel:</strong><br>' +
    'Ing. Jiří Hajda<br>' +
    'Zděchov 208<br>' +
    '75607 Zděchov<br>' +
    'IČ: 75904535<br>' +
    'DIČ: CZ8303255884<br>' +
    'Účet: 218098370 / 0300<br>' +
    'Neplátce DPH<br>' +
    '</td><td valign="top">' +
    '<strong>Odběratel:</strong><br>' +
    User.SubjectName + '<br>' +
    User.Street + '<br>' +
    User.PSC + ' ' + User.Town + '<br>';
  if User.IC <> 0 then Result := Result + 'IČ: ' + IntToStr(User.IC) + '<br>';
  if User.DIC  <> '' then Result := Result + 'DIČ: ' + User.DIC + '<br>';
  Result := Result + '</td></tr>' +
    '<tr><td colspan="2"><hr></td></tr>' +
    '<tr><td width="50%" valign="top">' +
    '<strong>Platební podmínky:</strong><br>' +
    'Číslo dokladu: ' + BillCode + '<br>' +
    'Variabilní symbol: ' + IntToStr(User.Id) + '<br>' +
    'Specifický symbol: ' + IntToStr(SpecificSymbol) + '<br>' +
    'Konstantní symbol:<br>' +
    'Způsob úhrady: převodem' +
    '</td><td valign="top">' +
    '<br>' +
    'Datum vystavení: ' + DateToStr(TimeCreate) + '<br>' +
    'Datum zdanitel. plnění: ' + DateToStr(TimeCreate) + '<br>' +
    'Datum splatnosti: ' + DateToStr(TimeDue) + '<br>';
  if(TimeFrom <> TDateTime(0)) and (TimeTo <> TDateTime(0)) then
    Result := Result + 'Fakturované období: ' + DateToStr(TimeFrom) + ' - ' + DateToStr(TimeTo) + '<br>';
  Result := Result + '</td></tr>' +
    '<tr><td colspan="2">' +
    '<hr>' +
    '<br>' +
    '<table border="0" width="100%">' +
    '<tr><th align="left">Dodávka</th><th align="right">Množství</th><th align="right">Cena/MJ</th><th align="right">Celkem</th></tr>' +
    '<tr><td colspan="4"><hr></td></tr>';
     Total := 0;
  for I := 0 to Items.Count - 1 do
  with TBillItem(Items[I]) do
  begin
    Result := Result + '<tr><td>' + Description + '</td><td align="right">' + IntToStr(Quantity) + '</td><td align="right">' + FloatToStr(Price)  +
      '&nbsp;Kč</td><td align="right">' + FloatToStr(Quantity * Price) + '&nbsp;Kč</td></tr>';
    Total := Total + Quantity * Price;
  end;
  Result := Result + '<tr><th colspan="3" align="left">Celkem</th><th align="right">' + FloatToStr(Total) + '&nbsp;Kč</th></tr>';
  Result := Result + '</table>' +
    '</td></tr>' +
    '<tr><td colspan="2"><hr></td></tr>' +
    '</table>';
end;

procedure TBill.Load;
begin
  Id := StrToInt(DbRow.Values['id']);
  BillCode := DbRow.Values['BillCode'];
  TimeDue := UnixToDateTime(StrToInt(DbRow.Values['time_due']));
  TimeFrom := UnixToDateTime(StrToInt(DbRow.Values['time_from']));
  TimeTo := UnixToDateTime(StrToInt(DbRow.Values['time_to']));
  TimeCreate := UnixToDateTime(StrToInt(DbRow.Values['time_create']));
  UserId := StrToInt(DbRow.Values['user_id']);
end;

procedure TBill.Store;
var
  Data: TAssocArray;
begin
  Data := TAssocArray.Create;
  Data.AddKeyValue('BillCode', BillCode);
  Data.AddKeyValue('time_due', IntToStr(DateTimeToUnix(TimeDue)));
  Data.AddKeyValue('time_from', IntToStr(DateTimeToUnix(TimeFrom)));
  Data.AddKeyValue('time_to', IntToStr(DateTimeToUnix(TimeTo)));
  Data.AddKeyValue('time_create', IntToStr(DateTimeToUnix(TimeCreate)));
  Data.AddKeyValue('user_id', BillCode);
  Database.Update('finance_bills', Data, 'id=' + IntToStr(Id));
  Data.Free;
  inherited Store;
end;

function TBill.CreateBill: Integer;
begin

// Database.Insert('finance_bills', Itemsarray('time_create' => TimeToMysqlDateTime(time()), 'user_id' => $UserId, 'time_from' => TimeToMysqlDateTime($TimeFrom), 'time_to' => TimeToMysqlDateTime($TimeTo), 'time_due' => TimeToMysqlDateTime($TimeFrom + 15*24*3600), 'BillCode' => $BillCode));
 (*
    $BillId = $Database->insert_id;
    foreach($Items as $Item)
    {
      $Database->insert('finance_bills_items', array('bill_id' => $BillId, 'description' => $Item['description'], 'price' => $Item['price'], 'quantity' => $Item['quantity']));
    }
    //Header('Content-Type: application/pdf');
    $PdfData = $this->HtmlToPdf($this->GenerateBill($BillId));
    //echo($PdfData);
    file_put_contents($this->FileFolder.'/'.$this->FileNamePrefix.$BillId.'.pdf', $PdfData);
    //$Database->query('UPDATE finance_bills SET pdf = 0x'.bin2hex($PdfData).' WHERE id='.$BillId);
    return($BillId);
    *)
end;
(*
  function RegeneratePDF($BillId)
  {
    global $Database;

    $PdfData = $this->HtmlToPdf($this->GenerateBill($BillId));
    //echo($this->FileFolder.'/'.$this->FileNamePrefix.$BillId.'.pdf');
    echo(file_put_contents($this->FileFolder.'/'.$this->FileNamePrefix.$BillId.'.pdf', $PdfData));
    //$Database->query('UPDATE finance_bills SET pdf = 0x'.bin2hex($PdfData).' WHERE id='.$BillId);
  }

  function ShowStoredBill($BillId)
  {
    global $Database;

    //$DbResult = $Database->select('finance_bills', 'pdf', 'id='.$BillId);
    //if($DbResult->num_rows == 1)
    //{
    //  $DbRow = $DbResult->fetch_array();
    $FileName = $this->FileFolder.'/'.$this->FileNamePrefix.$BillId.'.pdf';
    if(file_exists($FileName))
    {
      Header('Content-Type: application/pdf');
      Header('Content-Disposition: attachment; filename="'.$this->FileNamePrefix.$BillId.'.pdf"');
      echo(file_get_contents($FileName));
    } else echo('Faktura nenalezena');
  }

  function ShowGeneratedBill($BillId)
  {
    global $Database;

    Header('Content-Type: application/pdf');
    Header('Content-Disposition: attachment; filename="'.$this->FileNamePrefix.$BillId.'.pdf"');
    echo($this->HtmlToPdf($this->GenerateBill($BillId)));
  }

  function HasPDFFile($BillId)
  {
    return(file_exists($this->FileFolder.'/'.$this->FileNamePrefix.$BillId.'.pdf'));
  }

  function HtmlToPdf($HtmlCode)
  {
    $Output = shell_exec('echo "'.addslashes(from_utf8($HtmlCode)).'"|htmldoc --no-numbered --webpage --charset 8859-2 -t pdf -');
    return($Output);
  }

  function CustomGenerate()
  {
    global $Database;

    $DbResult = $Database->select('finance_operations', '*', '(date="2008-02-01" AND comment="Poplatek za měsíc Únor") OR '.
    '(date="2008-01-01" AND comment="Poplatek za měsíc Leden") OR (date="2007-12-01" AND comment="Poplatek za měsíc Prosinec") AND (bill_id = 0)');
    while($Row = $DbResult->fetch_array())
    {
      echo($Row['id']."<br>\n");
      $Time = MysqlDateToTime($Row['date']);
      $BillId = $this->CreateBill($Row['user'], array(array('description' => 'Poplatek za připojení k síti', 'price' => (-$Row['money']), 'quantity' => 1)), $Time, $Time + (date("t", time()) - 1) * 24 * 3600);
     $Database->update('finance_operations', 'id='.$Row['id'], array('bill_id' => $BillId));
    }
  }

  function ShowPage()
  {
    global $Database;
    if(array_key_exists('user', $_GET))
    {

      $DbResult = $Database->select('finance_bills', '*', 'user_id='.$_GET['user']);
      while($Item = $DbResult->fetch_array())
      {
        echo('<a href="?bill='.$Item['id'].'">faktura '.$Item['id'].'</a> <a href="?billpdf='.$Item['id'].'">Uložené PDF</a> <a href="?billpdf2='.$Item['id'].'">Generované PDF</a> <a href="?regenerate='.$Item['id'].'">Přegenerovat</a><br>');
      }
    } else
    if(array_key_exists('billpdf', $_GET))
    {
      $this->ShowStoredBill($_GET['billpdf']);
    } else
    if(array_key_exists('billpdf2', $_GET))
    {
      $this->ShowGeneratedBill($_GET['billpdf2']);
    } else
    if(array_key_exists('regenerate', $_GET))
    {
      $this->RegeneratePDF($_GET['regenerate']);
    } else
    if(array_key_exists('bill', $_GET))
    {
      echo($this->GenerateBill($_GET['bill']));
    } else
    if(array_key_exists('generate', $_GET))
    {
      $this->CreateBill(1, array(array('description' => 'Poplatek za připojení k síti', 'price' => 1000, 'quantity' => 1)), time(), time());
    } else
    {
      ShowHeader('Faktury', 'Faktury');
      echo('Faktury:<br>');
      $DbResult = $Database->select('users', '*, CONCAT(second_name," ", first_name)  as fullname', '1 ORDER BY fullname');
      while($User = $DbResult->fetch_array())
      {
        echo('<a href="?user='.$User['id'].'">'.$User['fullname'].'</a><br>');
      }
      ShowFooter();
    }
  }
}

$InvoiceGenerator = new InvoiceGenerator;

?>
*)

{ TBillItem }

procedure TBillItem.Load;
var
  DbRows: TDbRows;
begin
  DbRows := Database.Select(ClassName, '*', 'Id=' + IntToStr(Id));
  Quantity := StrToInt(DbRows[0].Values['quantity']);
  Description := DbRows[0].Values['description'];
  Price := StrToFloat(DbRows[0].Values['price']);
  DbRows.Free;
end;

procedure TBillItem.Store;
var
  DbRows: TDbRows;
  Data: TAssocArray;
begin
  Data := TAssocArray.Create;
  Data.AddKeyValue('Quantitiy', IntToStr(Quantity));
  Data.AddKeyValue('Description', Description);
  Data.AddKeyValue('Price', FloatToStr(Price));
  Database.Insert(ClassName, Data);
end;

end.

